@@ -1,25 +1,60 @@
-import csv
+# Copyright (c) 2021, Ethan Henderson
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# 1. Redistributions of source code must retain the above copyright notice, this
+#    list of conditions and the following disclaimer.
+#
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
+#
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 import datetime as dt
 import json
 import logging
 import os
 import time
-import typing as t
-from collections import defaultdict
 
 import requests
 from requests_oauthlib import OAuth2Session
 
-from analytix.errors import *
+from analytix.errors import (
+    HTTPError,
+    InvalidRequest,
+    InvalidScopes,
+    MissingOptionalComponents,
+)
 from analytix.iso import CURRENCIES
-from analytix.packages import *
+from analytix.packages import PANDAS_AVAILABLE
 from analytix.secrets import TOKEN_STORE, YT_ANALYTICS_TOKEN
-from analytix.youtube.analytics import *
-from analytix.youtube.analytics import verify
+from analytix.youtube.analytics import (
+    YOUTUBE_ANALYTICS_API_VERSION,
+    YOUTUBE_ANALYTICS_SCOPES,
+    verify,
+)
 
 if PANDAS_AVAILABLE:
     import pandas as pd
 
+log = logging.getLogger("analytix")
+
 
 class YouTubeAnalytics:
     """A client class to retrieve data from the YouTube Analytics API.
@@ -68,6 +103,8 @@ class YouTubeAnalytics:
 
         Args:
             path (str): The path to the secrets file.
+
+        Keyword Args:
             scopes (iterable[str] | str): The scopes to use. Defaults to
                 "all".
             **kwargs (Any): Additional arguments to pass to the
@@ -82,7 +119,7 @@ class YouTubeAnalytics:
             )
 
         with open(path, mode="r", encoding="utf-8") as f:
-            logging.debug("Secrets file loaded")
+            log.debug("Secrets file loaded")
             secrets = json.load(f)["installed"]
 
         scopes = cls._resolve_scopes(scopes)
@@ -100,6 +137,8 @@ class YouTubeAnalytics:
 
         Args:
             secrets (dict[str, dict[str, str]]): The secrets dictionary.
+
+        Keyword Args:
             scopes (iterable[str] | str): The scopes to use. Defaults to
                 "all".
             **kwargs (Any): Additional arguments to pass to the
@@ -120,7 +159,7 @@ class YouTubeAnalytics:
     @staticmethod
     def _resolve_scopes(scopes):
         if scopes == "all":
-            logging.debug(f"Scopes set to {YOUTUBE_ANALYTICS_SCOPES}")
+            log.debug(f"Scopes set to {YOUTUBE_ANALYTICS_SCOPES}")
             return YOUTUBE_ANALYTICS_SCOPES
 
         if not isinstance(scopes, (tuple, list, set)):
@@ -140,13 +179,13 @@ class YouTubeAnalytics:
                 + ", ".join(diff)
             )
 
-        logging.debug(f"Scopes set to {scopes}")
+        log.debug(f"Scopes set to {scopes}")
         return scopes
 
     @staticmethod
     def _get_token():
         if not os.path.isfile(TOKEN_STORE / YT_ANALYTICS_TOKEN):
-            logging.info("No token found; you will need to authorise")
+            log.info("No token found; you will need to authorise")
             return ""
 
         with open(
@@ -154,12 +193,12 @@ class YouTubeAnalytics:
         ) as f:
             data = json.load(f)
         if time.time() > data["expires"]:
-            logging.info(
+            log.info(
                 "Token found, but it has expired; you will need to authorise"
             )
             return ""
 
-        logging.info(
+        log.info(
             (
                 "Valid token found; analytix will use this, so you don't need "
                 "to authorise"
@@ -172,19 +211,19 @@ class YouTubeAnalytics:
         when needed, so you often don't need to call this unless you
         want to override the default behaviour.
 
-        Args:
+        Keyword Args:
             store_token (bool): Whether to store the token locally for
                 future uses. Note that tokens are only valid for an hour
                 before they expire. Defaults to True.
             force (bool): Whether to force an authorisation even when
-                authorisation credentials are still value. If this is
+                authorisation credentials are still valid. If this is
                 False, calls to this method won't do anything if the
                 client is already authorised. Defaults to False.
             **kwargs (Any): Additional arguments to pass when creating
                 the authorisation URL.
         """
         if self._token and not force:
-            logging.info("Client is already authorised! Skipping...")
+            log.info("Client is already authorised! Skipping...")
             return
 
         url, _ = self._session.authorization_url(
@@ -197,10 +236,10 @@ class YouTubeAnalytics:
             client_secret=self.secrets["client_secret"],
         )
         self._token = token["access_token"]
-        logging.info("Token retrieved")
+        log.info("Token retrieved")
 
         if not store_token:
-            logging.info("Not storing token, as instructed")
+            log.info("Not storing token, as instructed")
             return
 
         os.makedirs(TOKEN_STORE, exist_ok=True)
@@ -212,7 +251,7 @@ class YouTubeAnalytics:
                 f,
                 ensure_ascii=False,
             )
-            logging.info(f"Key stored in {TOKEN_STORE / YT_ANALYTICS_TOKEN}")
+            log.info(f"Key stored in {TOKEN_STORE / YT_ANALYTICS_TOKEN}")
 
     authorize = authorise
 
@@ -224,6 +263,8 @@ class YouTubeAnalytics:
         Args:
             start_date (datetime.date): The date from which data should
                 be collected from.
+
+        Keyword Args:
             end_date (datetime.date): The date to collect data to.
                 Defaults to :code:`datetime.date.today()`.
             metrics (iterable[str] | str): The metrics (or columns) to
@@ -268,7 +309,7 @@ class YouTubeAnalytics:
         start_index = kwargs.pop("start_index", 1)
         include_historical_data = kwargs.pop("include_historical_data", False)
 
-        logging.debug("Verifying options...")
+        log.debug("Verifying options...")
         if "7DayTotals" in dimensions or "30DayTotals" in dimensions:
             raise InvalidRequest(
                 "the '7DayTotals' and '30DayTotals' dimensions were "
@@ -335,7 +376,7 @@ class YouTubeAnalytics:
 
         if "month" in dimensions:
             if start_date.day != 1 or end_date.day != 1:
-                logging.warning(
+                log.warning(
                     "The start and end dates must be the first date of the "
                     "month when the 'month' dimension is passed. analytix "
                     "corrects this automatically for convenience, but "
@@ -345,9 +386,9 @@ class YouTubeAnalytics:
                 start_date = dt.date(start_date.year, start_date.month, 1)
                 end_date = dt.date(end_date.year, end_date.month, 1)
 
-        logging.debug("Determining report type...")
+        log.debug("Determining report type...")
         rtype = verify.rtypes.determine(dimensions, metrics, filters)()
-        logging.info(f"Report type determined as: {rtype}")
+        log.info(f"Report type determined as: {rtype}")
 
         if metrics == "all":
             metrics = tuple(rtype.metrics)
@@ -356,11 +397,11 @@ class YouTubeAnalytics:
                 "expected tuple, list, or set of metrics, "
                 f"got {type(metrics).__name__}"
             )
-        logging.debug("Using these metrics: " + ", ".join(metrics))
+        log.debug("Using these metrics: " + ", ".join(metrics))
 
-        logging.debug("Verifying report...")
+        log.debug("Verifying report...")
         rtype.verify(dimensions, metrics, filters, sort_by, max_results)
-        logging.debug("Verification complete")
+        log.debug("Verification complete")
 
         url = (
             "https://youtubeanalytics.googleapis.com/"
@@ -378,10 +419,10 @@ class YouTubeAnalytics:
             f"&sort={','.join(sort_by)}"
             f"&startIndex={start_index}"
         )
-        logging.debug(f"URL: {url}")
+        log.debug(f"URL: {url}")
 
         if not self._token:
-            logging.debug("Authorising...")
+            log.debug("Authorising...")
             self.authorise()
 
         with requests.get(
@@ -393,7 +434,7 @@ class YouTubeAnalytics:
             error = data["error"]
             raise HTTPError(f"{error['code']}: {error['message']}")
 
-        logging.info("Creating report...")
+        log.info("Creating report...")
         return YouTubeAnalyticsReport(f"{rtype}", data)
 
     def daily_analytics(self, of=None, since=None, last=28, metrics="all"):
@@ -402,7 +443,7 @@ class YouTubeAnalytics:
 
         .. versionadded:: 2.1
 
-        Args:
+        Keyword Args:
             of (str | None): A video ID. Pass None to include all
                 videos. Defaults to None.
             since (datetime.date | None): The date to start collecting
@@ -438,7 +479,7 @@ class YouTubeAnalytics:
 
         .. versionadded:: 2.1
 
-        Args:
+        Keyword Args:
             of (str | None): A video ID. Pass None to include all
                 videos. Defaults to None.
             since (datetime.date | None): The date to start collecting
@@ -467,9 +508,9 @@ class YouTubeAnalytics:
 
         return self.retrieve(
             _resolve_date(
-                since or dt.date.today() - dt.timedelta(days=last * 30)
+                since or dt.date.today() - dt.timedelta(days=last * 31)
             ),
-            _resolve_date(dt.date.today() - dt.timedelta(days=30)),
+            _resolve_date(dt.date.today() - dt.timedelta(days=31)),
             dimensions=("month",),
             metrics=metrics,
             filters={"video": of} if of else {},
@@ -481,7 +522,7 @@ class YouTubeAnalytics:
 
         .. versionadded:: 2.1
 
-        Args:
+        Keyword Args:
             since (datetime.date | None): The date to start collecting
                 data from. If this is None, analytix will fall back to
                 the :code:`last` kwarg. Defaults to None.
@@ -513,7 +554,7 @@ class YouTubeAnalytics:
 
         .. versionadded:: 2.1
 
-        Args:
+        Keyword Args:
             by (str): The metric to sort by. Note that not all metrics
                 are supported. Reports retrieved using this method will
                 be sorted in descending order regardless of whether you
@@ -607,6 +648,8 @@ class YouTubeAnalyticsReport:
 
         Args:
             path (str): The path to save the file to.
+
+        Keyword Args:
             indent (int): The amount of spaces to use as an indent.
                 Defaults to 4.
         """
@@ -621,6 +664,8 @@ class YouTubeAnalyticsReport:
 
         Args:
             path (str): The path to save the file to.
+
+        Keyword Args:
             delimiter (int): The delimiter to use to separate columns.
                 Defaults to a comma (,).
         """
@@ -628,7 +673,6 @@ class YouTubeAnalyticsReport:
             path += ".csv"
 
         with open(path, mode="w", encoding="utf-8") as f:
-            writer = csv.writer(f, delimiter=delimiter)
-            writer.writerow(self.columns)
-            for r in self.data["rows"]:
-                writer.writerow(r)
+            f.write(f"{delimiter.join(self.columns)}\n")
+            for row in self.data["rows"]:
+                f.write(f"{delimiter.join(f'{i}' for i in row)}\n")
